from typing import Optional, Literal, List, Iterable

from pydantic import parse_obj_as

from statina.adapter import StatinaAdapter
from statina.constants import SCOPES, sort_table
from statina.crud.utils import paginate
from statina.models.database import User


def get_users_text_query(query_string: str) -> dict:
    """Text search with regex, case insensitive"""
    return {
        "$or": [
            {"username": {"$regex": query_string, "$options": "i"}},
            {"email": {"$regex": query_string, "$options": "i"}},
        ]
    }


def users(
    adapter: StatinaAdapter,
) -> List[User]:
    """Return all users from the batch collection"""
    users: Iterable[dict] = adapter.user_collection.find()
    return parse_obj_as(List[User], list(users))


def query_users(
    adapter: StatinaAdapter,
    page_size: int = 0,
    page_num: int = 0,
    query_string: Optional[str] = "",
    role: Literal["", "admin", "unconfirmed", "inactive", "R", "RW"] = "",
    sort_key: Literal["added", "username", "email"] = "added",
    sort_direction: Literal["ascend", "descend"] = "ascend",
) -> List[User]:
    """
    Query users from the user collection.
    Pagination can be enabled with <page_size> and <page_num> options.
    No pagination enabled by default.
    """
    skip, limit = paginate(page_size=page_size, page_num=page_num)
    users: Iterable[dict] = (
        adapter.user_collection.find(
            {
                "$and": [
                    {"role": {"$in": [role or x for x in SCOPES]}},
                    get_users_text_query(query_string=query_string),
                ]
            }
        )
        .sort(sort_key, sort_table.get(sort_direction))
        .skip(skip)
        .limit(limit)
    )
    return parse_obj_as(List[User], list(users))


def count_query_users(
    adapter: StatinaAdapter,
    query_string: Optional[str] = "",
    role: Literal["", "admin", "unconfirmed", "inactive", "R", "RW"] = "",
) -> int:
    """Count all queried users from the user collection"""
    return adapter.user_collection.count_documents(
        filter={
            "$and": [
                {"role": {"$in": [role or x for x in SCOPES]}},
                get_users_text_query(query_string=query_string),
            ]
        }
    )


def user(
    adapter: StatinaAdapter, email: Optional[str] = None, user_name: Optional[str] = None
) -> Optional[User]:
    """Find user from user collection"""
    if email:
        raw_user: dict = adapter.user_collection.find_one({"email": email})
    elif user_name:
        raw_user: dict = adapter.user_collection.find_one({"username": user_name})
    else:
        raise SyntaxError("Have to use email or user_name")
    if not raw_user:
        return None
    return User(**raw_user)



import datetime
import logging
import secrets
from typing import List, Literal, Optional

from fastapi import APIRouter, Depends, Form, HTTPException, Query, Security
from fastapi.encoders import jsonable_encoder
from fastapi.security import OAuth2PasswordBearer, SecurityScopes, OAuth2PasswordRequestForm
from jose import JWTError, jwt
from pydantic import EmailStr
from sendmail_container import FormDataRequest
from starlette.background import BackgroundTasks
from starlette.responses import JSONResponse

import statina.crud.find.users
from statina.adapter import StatinaAdapter
from statina.API.external.api.api_v1.templates.email.account_activated import (
    ACTIVATION_MESSAGE_TEMPLATE,
)
from statina.API.external.api.api_v1.templates.email.admin_mail import ADMIN_MESSAGE_TEMPLATE
from statina.API.external.api.api_v1.templates.email.confirmation import (
    CONFIRMATION_MESSAGE_TEMPLATE,
)
from statina.API.external.api.deps import (
    get_password_hash,
    authenticate_user,
    create_access_token,
    get_user_scopes,
)
from statina.config import email_settings, get_nipt_adapter, settings
from statina.crud import update
from statina.crud.insert import insert_user
from statina.exeptions import credentials_exception, forbidden_access_exception
from statina.models.database import User
from statina.models.database.user import inactive_roles
from statina.models.server.auth import TokenData, Token
from statina.models.server.new_user import PaginatedUserResponse
from statina.tools.email import send_email

router = APIRouter(prefix="/v2")

oauth2_scheme = OAuth2PasswordBearer(
    tokenUrl="/v2/token",
    scopes={
        "R": "Read-only access to database",
        "RW": "Read and write access",
        "admin": "Admin permissions",
    },
)


LOG = logging.getLogger(__name__)


async def get_current_user(
    security_scopes: SecurityScopes,
    token: str = Depends(oauth2_scheme),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    try:
        payload = jwt.decode(token, settings.secret_key, algorithms=settings.algorithm)
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
        token_data = TokenData(username=username, scopes=payload.get("scopes", []))
    except JWTError:
        raise credentials_exception
    user = statina.crud.find.users.user(user_name=username, adapter=adapter)
    if user is None:
        raise credentials_exception
    for scope in security_scopes.scopes:
        if scope not in token_data.scopes:
            raise forbidden_access_exception
    return user


async def get_current_active_user(
    current_user: User = Security(get_current_user, scopes=["unconfirmed"])
):
    if current_user:
        return current_user
    raise HTTPException(status_code=401)


@router.post("/token", response_model=Token)
def login_for_access_token(form_data: OAuth2PasswordRequestForm = Depends()):
    """Creating a time delimited access token if the user is found in the database."""

    user: User = authenticate_user(form_data.username, form_data.password)
    if not user:
        raise credentials_exception
    access_token = create_access_token(form_data=form_data, username=user.username)
    return JSONResponse(
        content=jsonable_encoder(
            Token(
                access_token=access_token,
                token_type="bearer",
                username=user.username,
                email=user.email,
                scopes=get_user_scopes(username=user.username),
                role=user.role,
            )
        )
    )


@router.post(
    "/user/register",
    response_model=User,
    response_model_exclude={"hashed_password", "verification_hex"},
)
async def register_user(
    background_tasks: BackgroundTasks,
    email: EmailStr = Form(...),
    username: str = Form(...),
    password_repeated: str = Form(...),
    password: str = Form(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):

    if statina.crud.find.users.user(user_name=username, adapter=adapter):
        return JSONResponse(f"Username {username} already taken!", status_code=409)
    elif statina.crud.find.users.user(email=email, adapter=adapter):
        return JSONResponse(f"Email {email} already in use!", status_code=409)
    if not secrets.compare_digest(password, password_repeated):
        return JSONResponse(content="Password mismatch!", status_code=400)

    user = User(
        username=username,
        email=email,
        added=datetime.datetime.now(),
        role="unconfirmed",
        hashed_password=get_password_hash(password),
    )
    try:
        insert_user(adapter=adapter, user=user)
        confirmation_link = (
            f"{email_settings.website_uri}/{user.username}/" f"{user.verification_hex}"
        )
        email_form = FormDataRequest(
            sender_prefix=email_settings.sender_prefix,
            email_server_alias=email_settings.email_server_alias,
            request_uri=email_settings.mail_uri,
            recipients=user.email,
            mail_title="Verify your email",
            mail_body=CONFIRMATION_MESSAGE_TEMPLATE.format(
                website_uri=email_settings.website_uri,
                confirmation_link=confirmation_link,
                username=user.username,
            ),
        )
        background_tasks.add_task(send_email, email_form)
    except Exception as e:
        LOG.error(e)
        return JSONResponse(f"Could not register user", status_code=500)

    return JSONResponse(
        content=jsonable_encoder(
            user.dict(exclude={"hashed_password", "verification_hex"}),
            by_alias=False,
        )
    )


@router.get("/users", response_model=PaginatedUserResponse)
def users(
    page_size: Optional[int] = Query(5),
    page_num: Optional[int] = Query(0),
    role: Literal["", "admin", "unconfirmed", "inactive", "R", "RW"] = Query(""),
    query_string: Optional[str] = Query(""),
    sort_key: Literal["added", "username", "email"] = Query("added"),
    sort_direction: Literal["ascend", "descend"] = Query("ascend"),
    current_user: User = Security(get_current_active_user, scopes=["admin"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """Admin view with table of all users."""
    user_list: List[User] = statina.crud.find.users.query_users(
        adapter=adapter,
        page_size=page_size,
        query_string=query_string,
        role=role,
        sort_key=sort_key,
        sort_direction=sort_direction,
        page_num=page_num,
    )
    document_count = statina.crud.find.users.count_query_users(
        adapter=adapter, query_string=query_string, role=role
    )
    return JSONResponse(
        content=jsonable_encoder(
            PaginatedUserResponse(document_count=document_count, documents=user_list),
            by_alias=False,
        ),
        status_code=200,
    )


@router.patch("/user/{username}/validate")
async def validate_user_email(
    username: str,
    background_tasks: BackgroundTasks,
    verification_hex: str = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    update_user: User = statina.crud.find.users.user(user_name=username, adapter=adapter)
    if not update_user:
        return JSONResponse(content="No such user", status_code=404)
    if update_user.role != "unconfirmed":
        return JSONResponse(content="User already confirmed", status_code=400)
    if not secrets.compare_digest(update_user.verification_hex, verification_hex):
        return JSONResponse(content="URL is wrong or token has expired", status_code=401)
    try:
        update_user.role = "inactive"
        update.update_user(adapter=adapter, user=update_user)
        email_form = FormDataRequest(
            sender_prefix=email_settings.sender_prefix,
            email_server_alias=email_settings.email_server_alias,
            request_uri=email_settings.mail_uri,
            recipients=email_settings.admin_email,
            mail_title="New user request",
            mail_body=ADMIN_MESSAGE_TEMPLATE.format(
                username=update_user.username,
                user_email=update_user.email,
                website_uri=email_settings.website_uri,
            ),
        )
        background_tasks.add_task(send_email, email_form)
        return JSONResponse(content=f"New status: {update_user.role}", status_code=202)
    except Exception as e:
        LOG.error(e)
        return JSONResponse(content="Server could not verify user", status_code=500)


@router.put("/user/{username}/role")
async def update_user_role(
    username: str,
    background_tasks: BackgroundTasks,
    role: Literal[
        "unconfirmed",
        "inactive",
        "R",
        "RW",
        "admin",
    ] = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["admin"]),
):
    update_user: User = statina.crud.find.users.user(user_name=username, adapter=adapter)
    old_role = update_user.role
    update_user.role = role
    update.update_user(adapter=adapter, user=update_user)
    if old_role in inactive_roles and role not in inactive_roles:
        email_form = FormDataRequest(
            sender_prefix=email_settings.sender_prefix,
            email_server_alias=email_settings.email_server_alias,
            request_uri=email_settings.mail_uri,
            recipients=update_user.email,
            mail_title="Your account has been activated",
            mail_body=ACTIVATION_MESSAGE_TEMPLATE.format(
                website_uri=email_settings.website_uri, username=update_user.username
            ),
        )
        background_tasks.add_task(send_email, email_form)
    return JSONResponse(content="User updated", status_code=202)


@router.get(
    "/user/me", response_model=User, response_model_exclude={"hashed_password", "verification_hex"}
)
async def read_users_me(
    current_user: User = Security(get_current_active_user, scopes=["unconfirmed"])
):
    return JSONResponse(
        content=jsonable_encoder(
            current_user.dict(exclude={"hashed_password", "verification_hex"}),
            by_alias=False,
        ),
        status_code=200,
    )


@router.delete("/user/{username}")
async def delete_user(
    username: str,
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["admin"]),
):
    adapter.user_collection.delete_one({"username": username})
    LOG.info(f"Deleting user {username}")
    return JSONResponse(content=f"User {username} deleted", status_code=201)



from typing import Iterable, List, Optional, Literal

from pydantic import parse_obj_as

from statina.adapter import StatinaAdapter
from statina.constants import sort_table
from statina.crud.utils import paginate
from statina.models.database import DatabaseBatch


def get_batches_text_query(query_string: str) -> dict:
    """Text search with regex, case insensitive"""
    return {
        "$or": [
            {"batch_id": {"$regex": query_string, "$options": "i"}},
            {"comment": {"$regex": query_string, "$options": "i"}},
            {"Flowcell": {"$regex": query_string, "$options": "i"}},
        ]
    }


def batch(adapter: StatinaAdapter, batch_id: str) -> Optional[DatabaseBatch]:
    """Find one batch from the batch collection"""

    raw_batch: dict = adapter.batch_collection.find_one({"batch_id": batch_id})
    if not raw_batch:
        return None
    return DatabaseBatch(**raw_batch)


def batches(
    adapter: StatinaAdapter,
) -> List[DatabaseBatch]:
    """Return all batches from the batch collection"""
    raw_batches: Iterable[dict] = adapter.batch_collection.find()
    return parse_obj_as(List[DatabaseBatch], list(raw_batches))


def query_batches(
    adapter: StatinaAdapter,
    page_size: int = 0,
    page_num: int = 0,
    sort_direction: Optional[Literal["ascend", "descend"]] = "descend",
    sort_key: Optional[
        Literal["batch_id", "SequencingDate", "Flowcell", "comment"]
    ] = "SequencingDate",
    query_string: Optional[str] = "",
) -> List[DatabaseBatch]:
    """
    Query batches from the batch collection.
    Pagination can be enabled with <page_size> and <page_num> options.
    No pagination enabled by default.
    """
    skip, limit = paginate(page_size=page_size, page_num=page_num)
    raw_batches: Iterable[dict] = (
        adapter.batch_collection.find(get_batches_text_query(query_string=query_string))
        .sort(sort_key, sort_table.get(sort_direction))
        .skip(skip)
        .limit(limit)
    )
    return parse_obj_as(List[DatabaseBatch], list(raw_batches))


def count_query_batches(adapter: StatinaAdapter, query_string: Optional[str] = "") -> int:
    """Count all queried batches from the batch collection"""
    return adapter.batch_collection.count_documents(
        filter=get_batches_text_query(query_string=query_string)
    )



from datetime import datetime, timedelta
from typing import Optional


from fastapi.security import OAuth2PasswordRequestForm
from jose import jwt
from passlib.context import CryptContext

import statina
from statina.adapter.plugin import StatinaAdapter
from statina.config import get_nipt_adapter, settings
from statina.constants import SCOPES
from statina.models.database import User

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


def verify_password(plain_password: str, hashed_password: str) -> bool:
    return pwd_context.verify(plain_password, hashed_password)


def get_password_hash(password: str) -> str:
    return pwd_context.hash(password)


def authenticate_user(username: str, password: str) -> Optional[User]:
    adapter: StatinaAdapter = get_nipt_adapter()
    user: User = statina.crud.find.users.user(adapter=adapter, user_name=username)

    if user and verify_password(password, user.hashed_password):
        return user


def find_user(username: str) -> Optional[User]:
    adapter: StatinaAdapter = get_nipt_adapter()
    user: User = statina.crud.find.users.user(adapter=adapter, user_name=username)

    return user


def get_user_scopes(username: str) -> list:
    user_obj: Optional[User] = find_user(username=username)
    return SCOPES.get(user_obj.role, [])


def create_access_token(
    username: str,
    form_data: OAuth2PasswordRequestForm,
    expires_delta: Optional[timedelta] = timedelta(minutes=15),
) -> str:
    expire = datetime.utcnow() + timedelta(minutes=settings.access_token_expire_minutes)
    to_encode = {
        "sub": form_data.username,
        "scopes": get_user_scopes(username=username),
        "exp": expire,
    }
    return jwt.encode(to_encode, settings.secret_key, algorithm=settings.algorithm)



from typing import Optional

from fastapi import APIRouter, Depends, Security
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse

from statina.adapter.plugin import StatinaAdapter
from statina.API.v2.endpoints.user import get_current_active_user
from statina.config import get_nipt_adapter
from statina.crud.find.plots.statistics_plot_data import (
    get_last_batches,
    get_statistics_for_box_plot,
    get_statistics_for_scatter_plot,
)
from statina.models.database import User

router = APIRouter(prefix="/v2")


@router.get("/statistics")
def statistics(
    nr_batches: Optional[int] = 20,
    current_user: User = Security(get_current_active_user, scopes=["R"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """Statistics view."""

    scatter_plots = ["Stdev_13", "Stdev_18", "Stdev_21"]
    box_plots = [
        "Chr13_Ratio",
        "Chr18_Ratio",
        "Chr21_Ratio",
        "FF_Formatted",
        "DuplicationRate",
        "MappedReads",
        "GC_Dropout",
        "AT_Dropout",
        "Bin2BinVariance",
        "Library_nM",
    ]

    batches = get_last_batches(adapter=adapter, nr_of_batches=nr_batches)
    batch_ids = [batch.get("batch_id") for batch in batches]
    box_stat = get_statistics_for_box_plot(adapter=adapter, batches=batch_ids, fields=box_plots)
    scatter_stat = get_statistics_for_scatter_plot(batches=batches, fields=scatter_plots)
    return JSONResponse(
        content=jsonable_encoder(
            dict(
                ticks=list(range(nr_batches)),
                nr_batches=nr_batches,
                batch_ids=batch_ids,
                box_stat=box_stat,
                box_plots=box_plots,
                scatter_stat=scatter_stat,
                scatter_plots=scatter_plots,
            ),
            by_alias=False,
        )
    )



from fastapi import APIRouter
from starlette.responses import JSONResponse
from statina.constants import SCOPES

router = APIRouter(prefix="/v2")


@router.get("/")
def base():
    """Landing page"""
    return JSONResponse("Welcome to Statina!")


@router.get("/scopes")
def scopes():
    """API scopes"""
    return SCOPES



from fastapi import APIRouter
from starlette.responses import RedirectResponse


from statina.config import email_settings


router = APIRouter()


@router.get("/", deprecated=True)
def batches():
    """List of all batches"""
    return RedirectResponse(email_settings.website_uri)



from typing import Literal, Optional, List

from pydantic import BaseModel, validator, Field

from statina.API.external.constants import (
    SEX_CHROM_ABNORM,
    TRIS_CHROM_ABNORM,
    TRISOMI_TRESHOLDS,
    FF_TRESHOLDS,
)
from statina.constants import sample_status_options
from statina.models.database import DataBaseSample
from statina.models.server.plots.fetal_fraction_sex import x_get_y


class SampleWarning(BaseModel):
    fetal_fraction_preface: Literal["danger", "default", "warning"]
    fetal_fraction_y: Literal["danger", "default", "warning"]
    z_score_13: Literal["danger", "default", "warning"]
    z_score_18: Literal["danger", "default", "warning"]
    z_score_21: Literal["danger", "default", "warning"]
    x0: Literal["danger", "default", "warning"]
    xxx: Literal["danger", "default", "warning"]
    other: Literal["danger", "default", "warning"]
    xxy: Literal["danger", "default", "warning"]
    xyy: Literal["danger", "default", "warning"]


class Status(BaseModel):
    status: sample_status_options
    edited: str


class ZScore(BaseModel):
    z_score_13: str = Field(..., alias="13")
    z_score_18: str = Field(..., alias="18")
    z_score_21: str = Field(..., alias="21")
    z_score_x: str = Field(..., alias="x")

    class Config:
        allow_population_by_field_name = True


class FetalFraction(BaseModel):
    x: str
    y: str
    preface: str


class Include(BaseModel):
    include: Optional[bool] = False
    edited: Optional[str] = ""


class Statuses(BaseModel):
    status_13: Status = Field(..., alias="13")
    status_18: Status = Field(..., alias="18")
    status_21: Status = Field(..., alias="21")
    status_x0: Status = Field(..., alias="x0")
    status_xxx: Status = Field(..., alias="xxx")
    status_xxy: Status = Field(..., alias="xxy")
    status_xyy: Status = Field(..., alias="xyy")

    class Config:
        allow_population_by_field_name = True


class SampleValidator(DataBaseSample):
    warnings: Optional[SampleWarning]
    status_string: Optional[str]
    status: Optional[Statuses]
    fetal_fraction: Optional[FetalFraction]
    z_score: Optional[ZScore]
    included: Optional[Include]
    sex: Optional[Literal["XX", "XY"]]
    sample_type: str = Field(..., alias="SampleType")
    qc_flag: str = Field(..., alias="QCFlag")
    cnv_segment: Optional[str] = Field(..., alias="CNVSegment")
    text_warning: Optional[str]

    @validator("warnings", always=True)
    def set_warnings(cls, v, values: dict) -> SampleWarning:
        """Get warnings for a sample and return a SampleWarning"""

        sample_warnings = {}
        fetal_fraction_pf = values.get("FF_Formatted")
        fetal_fraction_y = values.get("FFY")
        fetal_fraction_x = values.get("FFX")
        sample_warnings["fetal_fraction_preface"]: str = cls.get_ff_preface_warning(
            fetal_fraction_pf=fetal_fraction_pf, fetal_fraction_y=fetal_fraction_y
        )
        sample_warnings["other"]: str = cls.get_other_warning(
            fetal_fraction_y=fetal_fraction_y, fetal_fraction_x=fetal_fraction_x
        )
        sample_warnings["fetal_fraction_y"]: str = cls.get_ff_y_warning(
            fetal_fraction_y=fetal_fraction_y
        )
        sample_warnings["x0"]: str = cls.get_x0_warning(
            fetal_fraction_y=fetal_fraction_y, fetal_fraction_x=fetal_fraction_x
        )
        sample_warnings["xxx"]: str = cls.get_XXX_warning(
            fetal_fraction_y=fetal_fraction_y, fetal_fraction_x=fetal_fraction_x
        )
        sample_warnings["xyy"]: str = cls.get_XYY_warning(
            fetal_fraction_y=fetal_fraction_y, fetal_fraction_x=fetal_fraction_x
        )
        sample_warnings["xxy"]: str = cls.get_XXY_warning(
            fetal_fraction_y=fetal_fraction_y, fetal_fraction_x=fetal_fraction_x
        )
        sample_warnings["z_score_13"]: str = cls.get_tris_warning(
            z_score=values.get("Zscore_13"),
            fetal_fraction_pf=fetal_fraction_pf,
            fetal_fraction_y=fetal_fraction_y,
        )
        sample_warnings["z_score_18"]: str = cls.get_tris_warning(
            z_score=values.get("Zscore_18"),
            fetal_fraction_pf=fetal_fraction_pf,
            fetal_fraction_y=fetal_fraction_y,
        )
        sample_warnings["z_score_21"]: str = cls.get_tris_warning(
            z_score=values.get("Zscore_21"),
            fetal_fraction_pf=fetal_fraction_pf,
            fetal_fraction_y=fetal_fraction_y,
        )
        return SampleWarning(**sample_warnings)

    @validator("fetal_fraction", always=True)
    def set_fetal_fraction(cls, v, values: dict) -> FetalFraction:

        return FetalFraction(
            x=round(values["FFX"], 2),
            y=round(values["FFY"], 2),
            preface=round(values["FF_Formatted"], 2),
        )

    @validator("z_score", always=True)
    def set_z_score(cls, v, values: dict) -> ZScore:

        return ZScore(
            z_score_13=round(values["Zscore_13"], 2),
            z_score_18=round(values["Zscore_18"], 2),
            z_score_21=round(values["Zscore_21"], 2),
            z_score_x=round(values["Zscore_X"], 2),
        )

    @validator("status_string", always=True)
    def set_status_string(cls, v, values: dict) -> str:

        status_list = []
        for key in TRIS_CHROM_ABNORM + SEX_CHROM_ABNORM:
            status = values.get(f"status_{key}")
            if status and status != "Normal":
                status_list.append(" ".join([status, key]))
        return ", ".join(status_list)

    @validator("status", always=True)
    def set_status(cls, v, values: dict) -> Statuses:

        statuses_dict = {
            f"status_{key.lower()}": Status(
                status=values.get(f"status_{key}"),
                edited=values.get(f"status_change_{key}"),
            )
            for key in TRIS_CHROM_ABNORM + SEX_CHROM_ABNORM
        }

        return Statuses(**statuses_dict)

    @validator("text_warning", always=True)
    def set_text_warning(cls, v, values: dict) -> str:
        """Joining the warnings for a sample to a text string"""
        if not values.get("warnings"):
            return ""
        text_warnings = [
            abn
            for abn, warning in values["warnings"].dict().items()
            if warning in ["danger", "warning"]
        ]
        return ", ".join(text_warnings)

    @validator("sex", always=True)
    def set_sex(cls, v, values: dict) -> str:
        """Set sex based on fetal fraction Y thresholds"""

        return "XY" if values.get("FFY") >= FF_TRESHOLDS["fetal_fraction_y_min"] else "XX"

    @validator("included", always=True)
    def set_include(cls, v, values: dict) -> Optional[Include]:
        return Include(include=values["include"], edited=values["change_include_date"])

    @classmethod
    def get_tris_warning(
        cls, z_score: float, fetal_fraction_pf: float, fetal_fraction_y: float
    ) -> str:
        """Get automated trisomy warning, based on preset Zscore thresholds"""
        hard_max = TRISOMI_TRESHOLDS["hard_max"]["Zscore"]
        soft_max = TRISOMI_TRESHOLDS["soft_max"]["Zscore"]
        hard_min = TRISOMI_TRESHOLDS["hard_min"]["Zscore"]
        preface_threshold = FF_TRESHOLDS["fetal_fraction_preface"]
        fetal_fraction_y_threshold = FF_TRESHOLDS["fetal_fraction_y_for_trisomy"]

        if fetal_fraction_pf is None or z_score is None or fetal_fraction_y is None:
            return "default"
        if z_score >= hard_max:
            return "danger"
        elif z_score >= soft_max and (
            (fetal_fraction_y < fetal_fraction_y_threshold and fetal_fraction_y != 0)
            or fetal_fraction_pf < preface_threshold
        ):
            return "warning"
        elif z_score <= hard_min:
            return "danger"
        return "default"

    @classmethod
    def get_ff_y_warning(cls, fetal_fraction_y: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        if not isinstance(fetal_fraction_y, (float, int)):
            return "default"

        if fetal_fraction_y < FF_TRESHOLDS["fetal_fraction_y_max"] and fetal_fraction_y != 0:
            return "danger"

        return "default"

    @classmethod
    def get_x0_warning(cls, fetal_fraction_y: float, fetal_fraction_x: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        x_treshold = FF_TRESHOLDS["fetal_fraction_X0"]
        y_treshold = FF_TRESHOLDS["fetal_fraction_y_min"]

        if not (
            isinstance(fetal_fraction_y, (float, int))
            and isinstance(fetal_fraction_x, (float, int))
        ):
            return "default"

        if fetal_fraction_y < y_treshold and fetal_fraction_x > x_treshold:
            return "danger"

        return "default"

    @classmethod
    def get_XXX_warning(cls, fetal_fraction_y: float, fetal_fraction_x: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        x_treshold = FF_TRESHOLDS["fetal_fraction_XXX"]
        y_treshold = FF_TRESHOLDS["fetal_fraction_y_min"]

        if not (
            isinstance(fetal_fraction_y, (float, int))
            and isinstance(fetal_fraction_x, (float, int))
        ):
            return "default"

        if fetal_fraction_y < y_treshold and fetal_fraction_x < x_treshold:
            return "danger"

        return "default"

    @classmethod
    def get_ff_preface_warning(cls, fetal_fraction_pf: float, fetal_fraction_y: float) -> str:
        """Get fetal fraction preface warning based on preset threshold"""

        if not isinstance(fetal_fraction_pf, (float, int)):
            return "default"
        if fetal_fraction_pf < FF_TRESHOLDS["fetal_fraction_preface"]:
            return "danger"
        return "default"

    @classmethod
    def get_other_warning(cls, fetal_fraction_y: float, fetal_fraction_x: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        k_lower: float = FF_TRESHOLDS["k_lower"]
        m_lower: float = FF_TRESHOLDS["m_lower"]
        y_treshold: float = FF_TRESHOLDS["fetal_fraction_y_min"]

        if not (
            isinstance(fetal_fraction_y, (float, int))
            and isinstance(fetal_fraction_x, (float, int))
        ):
            return "default"

        if y_treshold < fetal_fraction_y <= x_get_y(x=fetal_fraction_x, k=k_lower, m=m_lower):
            return "danger"

        return "default"

    @classmethod
    def get_XXY_warning(cls, fetal_fraction_y: float, fetal_fraction_x: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        k_upper: float = FF_TRESHOLDS["k_upper"]
        m_upper: float = FF_TRESHOLDS["m_upper"]
        x_threshold: float = FF_TRESHOLDS["fetal_fraction_X0"]

        if not (
            isinstance(fetal_fraction_y, (float, int))
            and isinstance(fetal_fraction_x, (float, int))
        ):
            return "default"

        if fetal_fraction_y > x_get_y(x=fetal_fraction_x, k=k_upper, m=m_upper) and (
            fetal_fraction_x <= x_threshold
        ):
            return "danger"
        return "default"

    @classmethod
    def get_XYY_warning(cls, fetal_fraction_y: float, fetal_fraction_x: float) -> str:
        """Get fetal fraction warning based on preset threshold"""

        k_upper: float = FF_TRESHOLDS["k_upper"]
        m_upper: float = FF_TRESHOLDS["m_upper"]
        x_treshold: float = FF_TRESHOLDS["fetal_fraction_X0"]

        if not (
            isinstance(fetal_fraction_y, (float, int))
            and isinstance(fetal_fraction_x, (float, int))
        ):
            return "default"

        if fetal_fraction_y > x_get_y(x=fetal_fraction_x, k=k_upper, m=m_upper) and (
            fetal_fraction_x > x_treshold
        ):
            return "danger"
        return "default"

    class Config:
        allow_population_by_field_name = True


class Sample(BaseModel):
    sample_type: str
    qc_flag: str
    cnv_segment: Optional[str]
    comment: str
    sample_id: str
    batch_id: str
    warnings: Optional[SampleWarning]
    text_warning: str
    sex: Optional[str]
    sequencing_date: Optional[str]
    status: Statuses
    included: Include
    z_score: ZScore
    fetal_fraction: FetalFraction

    class Config:
        allow_population_by_field_name = True


class PaginatedSampleResponse(BaseModel):
    document_count: int
    documents: List[Sample]



from typing import Optional, List

from pydantic import BaseModel


class Token(BaseModel):
    access_token: str
    token_type: str
    username: str
    email: str
    scopes: Optional[List]
    role: str


class TokenData(BaseModel):
    username: Optional[str] = None
    scopes: Optional[List[str]] = []



from typing import Literal

import pymongo

SCOPES = {
    "unconfirmed": ["unconfirmed"],
    "inactive": [
        "unconfirmed",
        "inactive",
    ],
    "R": ["unconfirmed", "inactive", "R"],
    "RW": ["unconfirmed", "inactive", "R", "RW"],
    "admin": ["unconfirmed", "inactive", "R", "RW", "admin"],
}


sort_table = {"ascend": pymongo.ASCENDING, "descend": pymongo.DESCENDING}
sample_status_options = Literal[
    "Normal",
    "False Positive",
    "Suspected",
    "Verified",
    "Probable",
    "False Negative",
    "Other",
    "Failed",
]

sample_sort_keys = Literal[
    "sample_id",
    "batch_id",
    "Zscore_13",
    "Zscore_18",
    "Zscore_21",
    "Zscore_X",
    "FF_Formatted",
    "CNVSegment",
    "FFY",
    "FFX",
    "comment",
    "QCFlag",
]



from typing import Optional, Literal, List, Iterable

from pydantic import parse_obj_as

from statina.adapter import StatinaAdapter
from statina.constants import sort_table, sample_sort_keys
from statina.crud.utils import paginate
from statina.models.database import DataBaseSample


def get_sample_text_query(query_string: str) -> dict:
    """Text search with regex, case insensitive"""
    return {
        "$or": [
            {"batch_id": {"$regex": query_string, "$options": "i"}},
            {"sample_id": {"$regex": query_string, "$options": "i"}},
            {"comment": {"$regex": query_string, "$options": "i"}},
            {"SampleType": {"$regex": query_string, "$options": "i"}},
            {"QCFlag": {"$regex": query_string, "$options": "i"}},
        ]
    }


def samples(
    adapter: StatinaAdapter,
) -> List[DataBaseSample]:
    """Return all samples from the sample collection"""
    raw_samples: Iterable[dict] = adapter.sample_collection.find()
    return parse_obj_as(List[DataBaseSample], list(raw_samples))


def query_samples(
    adapter: StatinaAdapter,
    batch_id: Optional[str] = None,
    sort_key: Optional[sample_sort_keys] = "sample_id",
    sort_direction: Optional[Literal["ascend", "descend"]] = "descend",
    query_string: Optional[str] = "",
    page_size: int = 0,
    page_num: int = 0,
) -> List[DataBaseSample]:
    """
    Query samples from the sample collection.
    Pagination can be enabled with <page_size> and <page_num> options.
    No pagination enabled by default.
    """
    skip, limit = paginate(page_size=page_size, page_num=page_num)
    raw_samples: Iterable[dict] = (
        adapter.sample_collection.find(
            {
                "$and": [
                    {"batch_id": batch_id or {"$regex": ".*"}},
                    get_sample_text_query(query_string=query_string),
                ],
            }
        )
        .sort(sort_key, sort_table.get(sort_direction))
        .skip(skip)
        .limit(limit)
    )
    return parse_obj_as(List[DataBaseSample], list(raw_samples))


def count_query_samples(
    adapter: StatinaAdapter, batch_id: Optional[str] = None, query_string: Optional[str] = ""
) -> int:
    """Count all queried samples in sample collection"""
    return adapter.sample_collection.count_documents(
        filter={
            "$and": [
                {"batch_id": batch_id or {"$regex": ".*"}},
                get_sample_text_query(query_string=query_string),
            ],
        }
    )


def sample(adapter: StatinaAdapter, sample_id: str) -> Optional[DataBaseSample]:
    """Find one sample from the sample collection"""

    raw_sample = adapter.sample_collection.find_one({"sample_id": sample_id})
    if not raw_sample:
        return None

    return DataBaseSample(**raw_sample)


def sample_aggregate(adapter: StatinaAdapter, pipe: list) -> list:
    """Aggregates a query pipeline on the sample collection"""

    return list(adapter.sample_collection.aggregate(pipe))


def batch_samples(
    adapter: StatinaAdapter,
    batch_id: str,
) -> List[DataBaseSample]:
    """All samples within the batch"""
    raw_samples: Iterable[dict] = adapter.sample_collection.find({"batch_id": batch_id})
    return parse_obj_as(List[DataBaseSample], list(raw_samples))


def query_batch_samples(
    adapter: StatinaAdapter,
    batch_id: str,
    page_size: int = 0,
    page_num: int = 0,
    sort_key: Optional[sample_sort_keys] = "sample_id",
    sort_direction: Optional[Literal["ascend", "descend"]] = "descend",
    query_string: Optional[str] = "",
) -> List[DataBaseSample]:
    """
    Query samples within the batch.
    Pagination can be enabled with <page_size> and <page_num> options.
    No pagination enabled by default.
    """
    skip, limit = paginate(page_size=page_size, page_num=page_num)
    raw_samples: Iterable[dict] = (
        adapter.sample_collection.find(
            {
                "$and": [{"batch_id": batch_id}, get_sample_text_query(query_string=query_string)],
            }
        )
        .sort(sort_key, sort_table.get(sort_direction))
        .skip(skip)
        .limit(limit)
    )
    return parse_obj_as(List[DataBaseSample], list(raw_samples))


def count_query_batch_samples(
    adapter: StatinaAdapter, batch_id: str, query_string: Optional[str] = ""
) -> int:
    """Count queried samples within the batch"""
    return adapter.sample_collection.count_documents(
        filter={
            "$and": [{"batch_id": batch_id}, get_sample_text_query(query_string=query_string)],
        }
    )



import io
from pathlib import Path
from typing import Tuple, Union
from zipfile import ZipFile, ZIP_DEFLATED


def paginate(page_size: int, page_num: int) -> Tuple[int, int]:
    """Calculate number of documents to skip"""
    if not page_size:
        return 0, 0
    if not page_num:
        return 0, page_size
    skip = page_size * (page_num - 1)
    return skip, page_size


def zip_dir(source_dir: Union[str, Path]) -> io.BytesIO:
    """Function for zipping"""
    src_path = Path(source_dir).expanduser().resolve(strict=True)
    len_src_path = len(src_path.as_posix())
    file_obj = io.BytesIO()
    file_obj.seek(0)
    with ZipFile(file=file_obj, mode="a", compression=ZIP_DEFLATED, compresslevel=9) as zf:
        for file in src_path.rglob("*"):
            zf.write(filename=file.as_posix(), arcname=file.as_posix()[len_src_path:])
    file_obj.seek(0)
    return file_obj



from pydantic import BaseModel


class Dataset(BaseModel):
    name: str
    fetal_fraction_preface: float = 4
    fetal_fraction_y_for_trisomy: float = 4
    fetal_fraction_y_max: float = 3
    fetal_fraction_y_min: float = 0.6
    fetal_fraction_XXX: float = -1
    fetal_fraction_X0: float = 3.4
    y_axis_min: float = -1
    y_axis_max: float = 20
    k_upper: float = 0.9809
    k_lower: float = 0.9799
    m_lower: float = -4.3987
    m_upper: float = 6.5958
    trisomy_soft_max: float = 3
    trisomy_hard_max: float = 4
    trisomy_hard_min: float = -8



from typing import List, Optional

from pydantic import BaseModel, Field, validator

from statina.models.database import DatabaseBatch


class Median(BaseModel):
    median_13: Optional[float] = Field(..., alias="13")
    median_18: Optional[float] = Field(..., alias="18")
    median_21: Optional[float] = Field(..., alias="21")
    median_x: Optional[float] = Field(..., alias="x")
    median_y: Optional[float] = Field(..., alias="y")

    class Config:
        allow_population_by_field_name = True


class Stdev(BaseModel):
    stdev_13: Optional[float] = Field(..., alias="13")
    stdev_18: Optional[float] = Field(..., alias="18")
    stdev_21: Optional[float] = Field(..., alias="21")
    stdev_x: Optional[float] = Field(..., alias="x")
    stdev_y: Optional[float] = Field(..., alias="y")

    class Config:
        allow_population_by_field_name = True


class BatchValidator(DatabaseBatch):
    flowcell: Optional[str] = Field(None, alias="Flowcell")
    sequencing_date: Optional[str] = Field(None, alias="SequencingDate")
    median: Optional[Median]
    stdev: Optional[Stdev]

    @validator("median", always=True)
    def set_median(cls, v, values: dict) -> Median:
        return Median(
            median_13=round(values["Median_13"], 2),
            median_18=round(values["Median_18"], 2),
            median_21=round(values["Median_21"], 2),
            median_x=round(values["Median_X"], 2),
            median_y=round(values["Median_Y"], 2),
        )

    @validator("stdev", always=True)
    def set_stdev(cls, v, values: dict) -> Stdev:
        return Stdev(
            stdev_13=round(values["Stdev_13"], 2),
            stdev_18=round(values["Stdev_18"], 2),
            stdev_21=round(values["Stdev_21"], 2),
            stdev_x=round(values["Stdev_X"], 2),
            stdev_y=round(values["Stdev_Y"], 2),
        )


class Batch(BaseModel):
    batch_id: str
    result_file: Optional[str]
    multiqc_report: Optional[str]
    segmental_calls: Optional[str]
    flowcell: Optional[str]
    sequencing_date: Optional[str]
    comment: Optional[str]
    median: Median
    stdev: Stdev

    class Config:
        validate_assignment = True
        allow_population_by_field_name = True


class PaginatedBatchResponse(BaseModel):
    document_count: int
    documents: List[Batch]



from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional

from fastapi import APIRouter, Depends, Form, Query, Security
from fastapi.encoders import jsonable_encoder
from fastapi.responses import FileResponse, JSONResponse

from statina.adapter import StatinaAdapter
from statina.API.v2.endpoints.user import get_current_active_user
from statina.config import get_nipt_adapter
from statina.constants import sample_status_options
from statina.crud import update
from statina.crud.find import samples as find_samples
from statina.crud.find import batches as find_batches
from statina.crud.find.plots import zscore_plot_data
from statina.models.database import DataBaseSample, User, DatabaseBatch
from statina.models.query_params import SamplesQuery
from statina.models.server.plots.ncv import Zscore131821, ZscoreSamples
from statina.models.server.sample import (
    Sample,
    PaginatedSampleResponse,
    SampleValidator,
)

from statina.parse.batch import validate_file_path

router = APIRouter(prefix="/v2")


@router.get("/samples", response_model=PaginatedSampleResponse)
def samples(
    sample_query: SamplesQuery = Depends(SamplesQuery),
    current_user: User = Security(get_current_active_user, scopes=["R"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """Get samples"""
    database_samples: List[DataBaseSample] = find_samples.query_samples(
        **sample_query.dict(), adapter=adapter
    )
    validated_samples: List[SampleValidator] = [
        SampleValidator(**sample.dict()) for sample in database_samples
    ]
    samples: List[Sample] = [Sample(**sample.dict()) for sample in validated_samples]

    document_count: int = find_samples.count_query_samples(
        adapter=adapter, batch_id=sample_query.batch_id, query_string=sample_query.query_string
    )
    return JSONResponse(
        content=jsonable_encoder(
            PaginatedSampleResponse(document_count=document_count, documents=samples), by_alias=True
        )
    )


@router.get("/sample/{sample_id}", response_model=Sample)
def sample(
    sample_id: str,
    current_user: User = Security(get_current_active_user, scopes=["R"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """Get sample with id"""

    database_sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    batch: DatabaseBatch = find_batches.batch(batch_id=database_sample.batch_id, adapter=adapter)

    if not database_sample:
        return JSONResponse("Not found", status_code=404)

    validated_sample = SampleValidator(**database_sample.dict())
    sample_view_data = Sample(
        sequencing_date=batch.SequencingDate,
        **validated_sample.dict(exclude_none=True),
    )

    return JSONResponse(
        content=jsonable_encoder(
            sample_view_data,
            by_alias=True,
        )
    )


@router.get("/sample/{sample_id}/tris")
def sample_tris(
    sample_id: str,
    current_user: User = Security(get_current_active_user, scopes=["R"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """Sample view with trisomi plot."""

    database_sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    abnormal_data: Dict[str, ZscoreSamples] = zscore_plot_data.get_abn_for_samp_tris_plot(
        adapter=adapter
    )
    normal_data: Zscore131821 = zscore_plot_data.get_normal_for_samp_tris_plot(adapter=adapter)
    sample_data: ZscoreSamples = zscore_plot_data.get_sample_for_samp_tris_plot(database_sample)

    return JSONResponse(
        content=jsonable_encoder(
            dict(
                normal_data=normal_data.dict(exclude_none=True, by_alias=True),
                abnormal_data=abnormal_data,
                sample_data=sample_data,
            ),
            by_alias=False,
        ),
    )


@router.put("/sample/{sample_id}/status/13")
async def set_sample_status_13(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_13 = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_13 = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/18")
async def set_sample_status_18(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_18 = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_18 = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/21")
async def set_sample_status_21(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_21 = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_21 = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/x0")
async def set_sample_status_x0(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_X0 = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_X0 = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/xxy")
async def set_sample_status_xxy(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_XXY = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_XXY = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/xxx")
async def set_sample_status_xxx(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_XXX = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_XXX = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/status/xyy")
async def set_sample_status_xyy(
    sample_id: str,
    status: sample_status_options = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update the manualy interpreted chromosome abnormality status for a sample."""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.status_XYY = status
    time_stamp: str = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    sample.status_change_XYY = f"{current_user.username} {time_stamp}"
    update.sample(adapter=adapter, sample=sample)
    return JSONResponse(content="Sample field updated", status_code=200)


@router.put("/sample/{sample_id}/comment")
async def sample_comment(
    sample_id: str,
    comment: Optional[str] = Form(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Update sample comment"""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.comment = comment
    update.sample(adapter=adapter, sample=sample)

    return JSONResponse(content="Sample comment updated", status_code=200)


@router.put("/sample/{sample_id}/include")
async def sample_include(
    sample_id: str,
    include: bool = Query(...),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
    current_user: User = Security(get_current_active_user, scopes=["RW"]),
):
    """Include sample in plots"""

    sample: DataBaseSample = find_samples.sample(sample_id=sample_id, adapter=adapter)
    sample.include = include
    update.sample(adapter=adapter, sample=sample)

    return JSONResponse(content="Sample inclusion status updated", status_code=200)


@router.get("/sample/{sample_id}/download/segmental_calls")
def sample_segmental_calls_download(
    sample_id: str,
    current_user: User = Security(get_current_active_user, scopes=["R"]),
    adapter: StatinaAdapter = Depends(get_nipt_adapter),
):
    """View for sample downloads"""

    sample: DataBaseSample = find_samples.sample(adapter=adapter, sample_id=sample_id)
    file_path = sample.dict().get("segmental_calls")
    if not validate_file_path(file_path):
        # warn file missing!
        JSONResponse(content="File missing on disk", status_code=404)

    file = Path(file_path)
    response = FileResponse(
        str(file.absolute()), media_type="application/octet-stream", filename=file.name
    )
    response.headers["Access-Control-Expose-Headers"] = "Content-Disposition"
    return response



from fastapi import APIRouter
from starlette.responses import RedirectResponse

from statina.config import email_settings

router = APIRouter()


@router.get("/", deprecated=True)
def index():
    """Log in view."""
    return RedirectResponse(email_settings.website_uri)



from typing import Optional, Literal

from pydantic import BaseModel

from statina.constants import sample_sort_keys


class ListQuery(BaseModel):
    page_size: Optional[int] = 5
    page_num: Optional[int] = 0
    sort_direction: Optional[Literal["ascend", "descend"]] = "descend"
    query_string: Optional[str] = ""
    sort_key: Optional[sample_sort_keys] = ""


class BatchesQuery(ListQuery):
    sort_key: Optional[
        Literal["batch_id", "SequencingDate", "Flowcell", "comment"]
    ] = "SequencingDate"


class BatchSamplesQuery(ListQuery):
    batch_id: str
    sort_key: Optional[sample_sort_keys] = "sample_id"


class SamplesQuery(ListQuery):
    batch_id: Optional[str] = None
    sort_key: Optional[sample_sort_keys] = "sample_id"
